
-- return the main font
-- the return is a list of font filename and size options
-- the first one found will be used
getFont = getFont or function()
	if isMac then
		return { { name="Verdana.ttf", size=13 }, { name="Monaco.dfont", size=13 } }
	else
		return { { name="Verdana.ttf", size=13 }, { name="arial.ttf", size=13 } }
	end
end
-- return the label font
getLabelFont = getLabelFont or function()
	if isMac then
		return { { name="Verdana.ttf", size=11 }, { name="Monaco.dfont", size=11 } }
	else
		return { { name="Verdana.ttf", size=11 }, { name="arial.ttf", size=11 } }
	end
end
-- return the text field font
getTextFieldFont = getTextFieldFont or function()
	if isMac then
		return { { name="Verdana.ttf", size=11 }, { name="Monaco.dfont", size=11 } }
	else
		return { { name="Verdana.ttf", size=11 }, { name="arial.ttf", size=11 } }
	end
end

-- knob sizes in the texture
kBitmapTinyKnobSize = 26
kBitmapKnobSize = 34
kBitmapBigKnobSize = 92

-- control sizing/layout globals
pad = pad or 10.0
windowEdgePad = windowEdgePad or 5.0
kTinyKnobSize = kTinyKnobSize or kBitmapTinyKnobSize
kKnobSize = kKnobSize or kBitmapKnobSize
kBigKnobSize = kBigKnobSize or kBitmapBigKnobSize
kTextFieldH = kTextFieldH or 12
kDropdownW = kDropdownW or 60

showTestSignals = showTestSignals or false
showLogicIndicators = showLogicIndicators or false
isLogicPro = isLogicPro or getBooleanProperty( 64140 )

setLabelColour = setLabelColour or function( lab )
	lab:setBackgroundColour( 0.1, 0.1, 0.15, 0.3 )
	lab:setForegroundColour( 0.9, 0.9, 0.9, 1.0 )
end

getGraphBackgroundColour = getGraphBackgroundColour or function()
	return 0.1, 0.1, 0.1, 1
end

getGraphFillColour = getGraphFillColour or function()
	return 0.15, 0.25, 0.5, 1
end

getGraphLineColour = getGraphLineColour or function()
	return 0.3, 0.5, 1, 1
end

setEnvelopeDisplayColours = setEnvelopeDisplayColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( getGraphFillColour() )
	env:setLineColour( getGraphLineColour() )
end

setCalibrationDisplayColours = setCalibrationDisplayColours or function( cb )
	cb:setBackgroundColour( getGraphBackgroundColour() )
	cb:setLineColour( getGraphLineColour() )
end

setWaveformDisplayColours = setWaveformDisplayColours or function( wv )
	wv:setBackgroundColour( getGraphBackgroundColour() )
	wv:setLineColour( getGraphLineColour() )
end

setFollowerEnvelopeDisplayColours = setFollowerEnvelopeDisplayColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( getGraphFillColour() )
	env:setLineColour( getGraphLineColour() )
	env:setPeakLineColour( 0.3, 0.8, 0.6, 1.0 )
end

setFollowerPitchDisplayColours = setFollowerPitchDisplayColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( getGraphFillColour() )
	env:setLineColour( getGraphLineColour() )
end

setStepEditorColours = setStepEditorColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( 0.2, 0.4, 0.8, 1 )
	env:setLineColour( getGraphLineColour() )
end

setFunctionDisplayColours = setFunctionDisplayColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setLineColour( getGraphLineColour() )
end

setRandomRegisterColours = setRandomRegisterColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( 0.2, 0.4, 0.8, 1 )
	env:setLineColour( getGraphLineColour() )
end

setDACEditorColours = setDACEditorColours or function( env )
	env:setBackgroundColour( getGraphBackgroundColour() )
	env:setFillColour( 0.2, 0.4, 0.8, 1 )
	env:setLineColour( getGraphLineColour() )
end

setUIColours = setUIColours or function()
	-- set window background & banner colours
	setBackgroundColour( 0.4, 0.4, 0.4 )
	setBannerColour( 0.1, 0.1, 0.15, 0.3 )
	setBannerTextColour( 0.9, 0.9, 0.9, 1.0 )

	-- set outline colours for groups
	setGroupActiveColour( 0.7, 0.7, 0.7, 1.0 )
	setGroupInactiveColour( 0.6, 0.6, 0.6, 1.0 )
end

knobAndTextField = knobAndTextField or function( group, param, labelText, x, y, size, textH, textW, format )
	textW = textW or size
	format = format or ""
	local knobx = x
	local textx = x
	if textW > size then
		knobx = knobx + 0.5 * ( textW - size )
	else
		textx = textx - 0.5 * ( textW - size )
	end
	group:add( knob( param, knobx, y+textH+1, size ) )
	group:add( textField( param, textx, y, textW, textH, format ) )
	if labelText then
		local lab = label( labelText, textx+0.5*textW, y-3 )
		setLabelColour( lab )
		group:add( lab )
	end
end

smallKnobAndTextField = smallKnobAndTextField or function( group, param, label, x, y, textW, format )
	format = format or ""
	knobAndTextField( group, param, label, x, y, kKnobSize, kTextFieldH, textW, format )
end

bigKnobAndTextField = bigKnobAndTextField or function( group, param, label, x, y )
	knobAndTextField( group, param, label, x, y, kBigKnobSize, kTextFieldH, kKnobSize, "" )
end

labelledDropDown = labelledDropDown or function( group, param, labelText, x, y, w )
	w = w or kDropdownW
	group:add( dropDown( param, x, y, w, kTextFieldH ) )
	local lab = label( labelText, x+0.5*w, y-3 )
	setLabelColour( lab )
	group:add( lab )
end

leftLabelledDropDown = leftLabelledDropDown or function( group, param, labelText, x, y, lw, w )
	w = w or kDropdownW
	group:add( dropDown( param, x+lw, y, w, kTextFieldH ) )
	local lab = label( labelText, x, y, "right", lw, kTextFieldH )
	setLabelColour( lab )
	group:add( lab )
end

createKnobRenderers = createKnobRenderers or function()
	-- load textures
	local atlas = texture( "buttons.png" )

	-- make renderers
	tinyKnobRenderer = arcKnobRenderer( atlas, 2, 128-100-kBitmapTinyKnobSize, kBitmapTinyKnobSize, 32, 128-100-kBitmapTinyKnobSize )
	smallKnobRenderer = arcKnobRenderer( atlas, 2, 128-2-kBitmapKnobSize, kBitmapKnobSize, 2, 128-42-kBitmapKnobSize )
	bigKnobRenderer = arcKnobRenderer( atlas, 52, 128-2-kBitmapBigKnobSize, kBitmapBigKnobSize, 162, 128-2-kBitmapBigKnobSize )
end

createUICommon = createUICommon or function()

	createKnobRenderers()

	setUIColours()

	local ww, wh = getWindowSize()
	local prefsW = kKnobSize+20
	local prefsH = 25
	local prefsX = ww-prefsW-windowEdgePad

	-- set location of the banner
	setBannerBounds( pad, wh - 25, prefsX - pad, wh - 10 )
	setBannerContent( "full", 60, -1 )

	-- add the prefs button
	
	add( button( "Prefs", "Prefs", prefsX, wh-prefsH-windowEdgePad, prefsW, prefsH ) )

end
